@@ -13,7 +13,7 @@ use vars qw ($VERSION $Debug);
 ######################################################################
 #### Configuration Section
 
-$VERSION = '3.310';
+$VERSION = '3.403';
 
 #######################################################################
 # CONSTRUCTORS
@@ -80,13 +80,12 @@ sub _read_split_file {
 	    }
 	}
 
-	(my $l2 = $line) =~ s!//.*$!!;
-	if ($l2 =~ s!.*?\*/!!) {
-	    $commented = 0;
-	}
-	while ($l2 =~ s!.*?/\*!!) {
-	    $commented = 1;
-	    if ($l2 =~ s!.*?\*/!!) {
+	while ($line =~ m!.*?(/\*|//|\*/)!g) {
+	    if (!$commented && $1 eq '//') {
+		last;
+	    } elsif (!$commented && $1 eq '/*') {
+		$commented = 1;
+	    } elsif ($commented && $1 eq '*/') {
 		$commented = 0;
 	    }
 	}
@@ -383,7 +382,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2006-2011 by Wilson Snyder.  This package is free software; you
+Copyright 2006-2014 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
