@@ -11,7 +11,7 @@ use base qw(DynaLoader);
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '3.314';
+$VERSION = '3.403';
 
 ######################################################################
 #### Configuration Section
@@ -36,6 +36,7 @@ sub new {
     my $self = {keep_comments=>1,
 		keep_whitespace=>1,
 		line_directives=>1,
+		ieee_predefined=>1,
 		pedantic=>0,
 		synthesis=>0,
 		options=>Verilog::Getopt->new(),	# If the user didn't give one, still work!
@@ -53,7 +54,25 @@ sub new {
 		$self->{synthesis},
 		);
     if ($self->{synthesis}) {
-	$self->define('SYNTHESIS',1);
+	# Fourth argument 1 for cmdline - no `undefineall effect
+	$self->define('SYNTHESIS',1,undef,1);
+    }
+    if ($self->{ieee_predefined}) {
+	$self->define('SV_COV_START', 0,undef,1);
+	$self->define('SV_COV_STOP', 1,undef,1);
+	$self->define('SV_COV_RESET', 2,undef,1);
+	$self->define('SV_COV_CHECK', 3,undef,1);
+	$self->define('SV_COV_MODULE', 10,undef,1);
+	$self->define('SV_COV_HIER', 11,undef,1);
+	$self->define('SV_COV_ASSERTION', 20,undef,1);
+	$self->define('SV_COV_FSM_STATE', 21,undef,1);
+	$self->define('SV_COV_STATEMENT', 22,undef,1);
+	$self->define('SV_COV_TOGGLE', 23,undef,1);
+	$self->define('SV_COV_OVERFLOW', -2,undef,1);
+	$self->define('SV_COV_ERROR', -1,undef,1);
+	$self->define('SV_COV_NOCOV', 0,undef,1);
+	$self->define('SV_COV_OK', 1,undef,1);
+	$self->define('SV_COV_PARTIAL', 2,undef,1);
     }
     #use Data::Dumper; print Dumper($self);
     return $self;
@@ -214,8 +233,7 @@ Verilog::Preproc - Preprocess Verilog files
 =head1 DESCRIPTION
 
 Verilog::Preproc reads Verilog files, and preprocesses them according to
-the SystemVerilog 2009 (1800-2009) specification which is the most recent
-extension of SystemVerilog 2005 (1800-2005).  Programs can be easily
+the SystemVerilog 2009 (1800-2009) specification.  Programs can be easily
 converted from reading a IO::File into reading preprocessed output from
 Verilog::Preproc.
 
@@ -259,9 +277,10 @@ that may be passed to new.
 
 =item $self->open(filename=>I<filename>)
 
-Opens the specified file.  If called before a file is completely parsed,
-the new file will be parsed completely before returning to the previously
-open file.  (As if it was an include file.)
+Opens the specified file.  If filename ends in .gz, decompress while
+reading.  If called before a file is completely parsed, the new file will
+be parsed completely before returning to the previously open file.  (As if
+it was an include file.)
 
 Open may also be called without named parameters, in which case the only
 argument is the filename.
@@ -280,6 +299,11 @@ The following named parameters may be passed to the new constructor.
 
 =over 4
 
+=item ieee_predefines=>0
+
+With ieee_predefines false, disable defining SV_COV_START and other IEEE
+mandated definitions.
+
 =item include_open_nonfatal=>1
 
 With include_open_nonfatal set to one, ignore any include files that do
@@ -386,10 +410,11 @@ after resolving the filename with the options parameter.
 
 =head1 COMPLIANCE
 
-The preprocessor supports the constructs defined in the SystemVerilog 2009
-standard (IEEE 1800-2009), which is a superset of Verilog 1995 (IEEE
+The preprocessor supports the constructs defined in the SystemVerilog 2012
+standard (IEEE 1800-2012), which is a superset of Verilog 1995 (IEEE
 1364-1995), Verilog 2001 (IEEE 1364-2001), Verilog 2005 (IEEE 1364-2005)
-and SystemVerilog 2005 (IEEE 1800-2005).
+and SystemVerilog 2005 (IEEE 1800-2005), and SystemVerilog 2009 (IEEE
+1800-2009).
 
 Verilog::Preproc adds the `error macro (unless the pedantic parameter is
 set.):
@@ -422,7 +447,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2012 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2014 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
