@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //*************************************************************************
 //
-// Copyright 2000-2010 by Wilson Snyder.  This program is free software;
+// Copyright 2000-2014 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -157,6 +157,10 @@ struct VPreProcImp : public VPreProcOpaque {
 	m_finAtBol = true;
 	m_defDepth = 0;
 	m_defPutJoin = false;
+	m_finToken = 0;
+	m_finFilelinep = NULL;
+	m_lexp = NULL;
+	m_preprocp = NULL;
     }
     void configure(VFileLine* filelinep, VPreProc* preprocp) {
 	// configure() separate from constructor to avoid calling abstract functions
@@ -167,6 +171,7 @@ struct VPreProcImp : public VPreProcOpaque {
 	m_lexp->m_keepComments = m_preprocp->keepComments();
 	m_lexp->m_keepWhitespace = m_preprocp->keepWhitespace();
 	m_lexp->m_pedantic = m_preprocp->pedantic();
+	m_lexp->m_synthesis = m_preprocp->synthesis();
 	m_lexp->debug(debug()>=10 ? debug() : 0);  // See also VPreProc::debug() method
     }
     ~VPreProcImp() {
@@ -201,7 +206,7 @@ private:
     void parsingOff() { m_off++; }
 
     int getRawToken();
-    int getStateToken();
+    int getStateToken(string& buf);
     int getFinalToken(string& buf);
 
     void statePush(ProcState state) {
@@ -226,6 +231,12 @@ private:
 VPreProc::VPreProc() {
     VPreProcImp* idatap = new VPreProcImp();
     m_opaquep = idatap;
+    // Below overridden by configure()
+    m_keepComments = true;
+    m_keepWhitespace = true;
+    m_lineDirectives = true;
+    m_pedantic = false;
+    m_synthesis = false;
 }
 
 void VPreProc::configure(VFileLine* filelinep) {
@@ -443,16 +454,18 @@ string VPreProcImp::defineSubst(VPreDefRef* refp) {
 	string argName;
 	string prev;
 	bool quote = false;
+	bool backslashesc = false;  // In \.....{space} block
 	// Note we go through the loop once more at the NULL end-of-string
 	for (const char* cp=value.c_str(); (*cp) || argName!=""; cp=(*cp?cp+1:cp)) {
 	    //cout << "CH "<<*cp<<"  an "<<argName<<"\n";
-	    if (!quote) {
-		if ( isalpha(*cp) || *cp=='_'
-		     || *cp=='$' // Won't replace system functions, since no $ in argValueByName
-		     || (argName!="" && (isdigit(*cp) || *cp=='$'))) {
-		    argName += *cp;
-		    continue;
-		}
+	    if (!quote && *cp == '\\') { backslashesc = true; }
+	    else if (isspace(*cp)) { backslashesc = false; }
+	    // We don't check for quotes; some simulators expand even inside quotes
+	    if ( isalpha(*cp) || *cp=='_'
+		 || *cp=='$' // Won't replace system functions, since no $ in argValueByName
+		 || (argName!="" && (isdigit(*cp) || *cp=='$'))) {
+		argName += *cp;
+		continue;
 	    }
 	    if (argName != "") {
 		// Found a possible variable substitution
@@ -469,7 +482,11 @@ string VPreProcImp::defineSubst(VPreDefRef* refp) {
 	    if (!quote) {
 		// Check for `` only after we've detected end-of-argname
 		if (cp[0]=='`' && cp[1]=='`') {
-		    out += "``";   // `` must get removed later, as `FOO```BAR must pre-expand FOO and BAR
+		    if (backslashesc) {
+			// Don't put out the ``, we're forming an escape which will not expand further later
+		    } else {
+			out += "``";   // `` must get removed later, as `FOO```BAR must pre-expand FOO and BAR
+		    }
 		    cp++;
 		    continue;
 		}
@@ -500,12 +517,18 @@ string VPreProcImp::defineSubst(VPreDefRef* refp) {
 		    continue;
 		}
 	    }
-	    if (cp[0]=='\\' && cp[1]) {
-		out += cp[0]; // \{any} Put out literal next character, including the escape
+	    if (cp[0]=='\\' && cp[1]=='\"') {
+		out += cp[0]; // \{any} Put out literal next character
 		out += cp[1];
 		cp++;
 		continue;
 	    }
+	    else if (cp[0]=='\\') {
+		// Normally \{any} would put out literal next character
+		// Instead we allow "`define A(nm) \nm" to expand, per proposed mantis1537
+		out += cp[0];
+		continue;
+	    }
 	    if (*cp=='"') quote=!quote;
 	    if (*cp) out += *cp;
 	}
@@ -519,14 +542,27 @@ string VPreProcImp::defineSubst(VPreDefRef* refp) {
 // Parser routines
 
 bool VPreProcImp::readWholefile(const string& filename, StrList& outl) {
-    int fd = open (filename.c_str(), O_RDONLY);
-    if (!fd) return false;
-
-    // If change this code, run a test with the below size set very small
+// If change this code, run a test with the below size set very small
 //#define INFILTER_IPC_BUFSIZ 16
 #define INFILTER_IPC_BUFSIZ 64*1024
     char buf[INFILTER_IPC_BUFSIZ];
+
+    FILE* fp = NULL;
+    int fd;
+    char cmd[100];
     bool eof = false;
+
+    ssize_t position = filename.find_last_of(".");
+    if (filename.length()>3 && 0==filename.compare(filename.length()-3, 3, ".gz")) {
+        sprintf(cmd, "gunzip -c %s", filename.c_str());
+        if ((fp = popen(cmd, "r")) == NULL) {
+            return false;
+        }
+        fd = fileno (fp);
+    } else {
+        fd = open (filename.c_str(), O_RDONLY);
+        if (fd<0) return false;
+    }
     while (!eof) {
 	ssize_t todo = INFILTER_IPC_BUFSIZ;
 	ssize_t got = read (fd, buf, todo);
@@ -541,7 +577,8 @@ bool VPreProcImp::readWholefile(const string& filename, StrList& outl) {
 	} else { eof = true; break; }
     }
 
-    close(fd);
+    if (fp) { pclose(fp); fp=NULL; }
+    else close(fd);
     return true;
 }
 
@@ -680,25 +717,37 @@ void VPreProcImp::debugToken(int tok, const char* cmtp) {
 // Sorry, we're not using bison/yacc. It doesn't handle returning white space
 // in the middle of parsing other tokens.
 
-int VPreProcImp::getStateToken() {
+int VPreProcImp::getStateToken(string& buf) {
     // Return the next state-determined token
     while (1) {
       next_tok:
-	if (isEof()) return VP_EOF;
+	if (isEof()) {
+	    buf = string (yyourtext(), yyourleng());
+	    return VP_EOF;
+	}
 	int tok = getRawToken();
 	ProcState state = m_states.top();
 
 	// Most states emit white space and comments between tokens. (Unless collecting a string)
-	if (tok==VP_WHITE && state !=ps_STRIFY) return (tok);
+	if (tok==VP_WHITE && state !=ps_STRIFY) {
+	    buf = string (yyourtext(), yyourleng());
+	    return (tok);
+	}
 	if (tok==VP_BACKQUOTE && state !=ps_STRIFY) { tok = VP_TEXT; }
 	if (tok==VP_COMMENT) {
-	    if (!m_off && m_lexp->m_keepComments) {
+	    if (!m_off) {
 		if (m_lexp->m_keepComments == KEEPCMT_SUB
 		    || m_lexp->m_keepComments == KEEPCMT_EXP) {
 		    string rtn; rtn.assign(yyourtext(),yyourleng());
 		    m_preprocp->comment(rtn);
-		} else {
+		    // Need to insure "foo/**/bar" becomes two tokens
+		    insertUnreadback (" ");
+		} else if (m_lexp->m_keepComments) {
+		    buf = string (yyourtext(), yyourleng());
 		    return (tok);
+		} else {
+		    // Need to insure "foo/**/bar" becomes two tokens
+		    insertUnreadback (" ");
 		}
 	    }
 	    // We're off or processed the comment specially.  If there are newlines
@@ -803,7 +852,10 @@ int VPreProcImp::getStateToken() {
 	    }
 	    else if (tok==VP_TEXT) {
 		// IE, something like comment between define and symbol
-		if (!m_off) return tok;
+		if (!m_off) {
+		    buf = string (yyourtext(), yyourleng());
+		    return tok;
+		}
 		else goto next_tok;
 	    }
 	    else if (tok==VP_DEFREF) {
@@ -824,7 +876,10 @@ int VPreProcImp::getStateToken() {
 		goto next_tok;
 	    } else if (tok==VP_TEXT) {
 		// IE, something like comment in formals
-		if (!m_off) return tok;
+		if (!m_off) {
+		    buf = string (yyourtext(), yyourleng());
+		    return tok;
+		}
 		else goto next_tok;
 	    } else {
 		error((string)"Expecting define formal arguments. Found: "+tokenName(tok)+"\n");
@@ -870,7 +925,7 @@ int VPreProcImp::getStateToken() {
 	    statePop();
 	    // DEFVALUE is terminated by a return, but lex can't return both tokens.
 	    // Thus, we emit a return here.
-	    yyourtext(newlines.c_str(), newlines.length());
+	    buf = newlines;
 	    return(VP_WHITE);
 	}
 	case ps_DEFPAREN: {
@@ -1101,6 +1156,7 @@ int VPreProcImp::getStateToken() {
 		if (m_off) {
 		    goto next_tok;
 		} else {
+		    buf = string (yyourtext(), yyourleng());
 		    return (VP_TEXT);
 		}
 	    }
@@ -1145,6 +1201,7 @@ int VPreProcImp::getStateToken() {
 	    if (!m_ifdefStack.empty()) {
 		error("`ifdef not terminated at EOF\n");
 	    }
+	    buf = string (yyourtext(), yyourleng());
 	    return tok;
 	case VP_UNDEFINEALL:
 	    if (!m_off) {
@@ -1163,7 +1220,10 @@ int VPreProcImp::getStateToken() {
 	case VP_PSL:
 	case VP_TEXT: {
 	    m_defDepth = 0;
-	    if (!m_off) return tok;
+	    if (!m_off) {
+		buf = string (yyourtext(), yyourleng());
+		return tok;
+	    }
 	    else goto next_tok;
 	}
 	case VP_WHITE:		// Handled at top of loop
@@ -1174,6 +1234,7 @@ int VPreProcImp::getStateToken() {
 	    fatalSrc((string)"Internal error: Unexpected token "+tokenName(tok)+"\n");
 	    break;
 	}
+	buf = string (yyourtext(), yyourleng());
 	return tok;
     }
 }
@@ -1183,8 +1244,7 @@ int VPreProcImp::getFinalToken(string& buf) {
     // Includes and such are handled here, and are never seen by the caller.
     if (!m_finAhead) {
 	m_finAhead = true;
-	m_finToken = getStateToken();
-	m_finBuf = string (yyourtext(), yyourleng());
+	m_finToken = getStateToken(m_finBuf);
     }
     int tok = m_finToken;
     buf = m_finBuf;
